window.onload = function(){
  var canvas = document.getElementById('canvas');
  var ctx = canvas.getContext('2d');

  function Particle(state) {
    this.x = 10;
    this.y = 10;
    this.xSpeed = Math.random() * 2.5;
    this.width = 5;
    this.height = 5;
    this.active = state;
    this.toRemove = false;
    this.lifeTime = 30;
    this.aliveFor = 0;
    this.update = function(){
      this.y += 4.5;
      this.x += this.xSpeed;
      this.aliveFor++;
      if(this.aliveFor>this.lifeTime) {
        this.toRemove = true;
      }
    }
    this.reset = function(){
      this.x = 10;
      this.y = 10;
      this.xSpeed = Math.random() * 2.5;
      this.active = false;
      this.aliveFor = 0;
      this.toRemove = false;
    }
  }

  var ParticlePool = {
    baseSize: 0,
    sizeIncrement: 10,
    particles: [],
    indexes: new DoublyLinkedList(),
    grow: function(){
      var newSize = this.baseSize + this.sizeIncrement;
      for (var i = this.baseSize; i < newSize; i++) {
        this.particles.push(new Particle(false))
        this.indexes.append(i);
      }
      this.baseSize = newSize;
    },
    spawnParticle: function(){
      if(this.indexes.head === null) {
        this.grow();
      }
      var index = this.indexes.viewAt(0);
      this.particles[index].active = true;
      this.indexes.removeAt(0);
      console.log(this.indexes.view());
      console.log(this.baseSize);
    },
    recycle: function(index){
      this.particles[index].reset();
      this.indexes.append(index);
    }
  };

  timer = 0;

  function handleTimer(){
    timer++;
    if(timer > 2) {
      timer = 0;
    }
  }

  function theLoop() {
    update();
    render();
    requestAnimationFrame(theLoop);
  }

  function update(){
    handleTimer();
    if(timer === 2) {
      ParticlePool.spawnParticle();
    }
    for (var i = 0; i < ParticlePool.particles.length; i++) {
      var part = ParticlePool.particles[i];
      if(part.toRemove){
        ParticlePool.recycle(i);
      }
      if(part.active){
        part.update();
      }
    }
  }

  function render(){
    ctx.clearRect(0,0,300,300)
    for (var i = 0; i < ParticlePool.particles.length; i++) {
      var part = ParticlePool.particles[i];
      if(part.active){
        ctx.fillRect(part.x,part.y,part.width,part.height);
      }
    }
  }

  theLoop();
}
